from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.uid import UID
from kfsd.apps.endpoints.serializers.common.uid import UIDViewModelSerializer
from kfsd.apps.endpoints.views.docs.uid import UIDDoc


@extend_schema_view(**UIDDoc.modelviewset())
class UIDModelViewSet(CustomModelViewSet):
    queryset = UID.objects.all()
    serializer_class = UIDViewModelSerializer



from .tables.outpost import Outpost
from .tables.uid import UID  # user
from .tables.oid import OID  # org
from .tables.pid import PID  # project
from .tables.tid import TID  # team
from .tables.cid import CID  # collection



from django.core.management.base import BaseCommand
from kfsd.apps.core.msmq.rabbitmq.base import RabbitMQ
from kfsd.apps.core.utils.time import Time
from kfsd.apps.endpoints.serializers.common.outpost import MsgSerializer
from kfsd.apps.endpoints.serializers.common.uid import UIDViewModelSerializer
from kfsd.apps.endpoints.serializers.common.oid import OIDViewModelSerializer
from kfsd.apps.endpoints.serializers.common.pid import PIDViewModelSerializer
from kfsd.apps.endpoints.serializers.common.tid import TIDViewModelSerializer
from kfsd.apps.endpoints.serializers.common.cid import CIDViewModelSerializer


from kfsd.apps.core.common.logger import Logger, LogLevel
from kfsd.apps.core.utils.dict import DictUtils
from kfsd.apps.models.tables.outpost import Outpost, send_msg
from kfsd.apps.models.constants import (
    OUTPOST_ACTION_CLEAR,
    UID_ACTION_CREATE,
    OID_ACTION_CREATE,
    PID_ACTION_CREATE,
    TID_ACTION_CREATE,
    CID_ACTION_CREATE,
    UID_ACTION_UPDATE,
    OID_ACTION_UPDATE,
    PID_ACTION_UPDATE,
    TID_ACTION_UPDATE,
    CID_ACTION_UPDATE,
    UID_ACTION_DELETE,
    OID_ACTION_DELETE,
    PID_ACTION_DELETE,
    TID_ACTION_DELETE,
    CID_ACTION_DELETE,
)

from kfsd.apps.models.tables.uid import UID
from kfsd.apps.models.tables.oid import OID
from kfsd.apps.models.tables.tid import TID
from kfsd.apps.models.tables.pid import PID
from kfsd.apps.models.tables.cid import CID


import json

logger = Logger.getSingleton(__name__, LogLevel.DEBUG)


def clear_outpost(data):
    outpostQS = Outpost.objects.filter(status="E").order_by("created")
    identifiers = [outpost.identifier for outpost in outpostQS]
    logger.info(
        "Recd CLEAR_OUTPOST command on msmq, identifiers: {} with 'Error' status found.".format(
            identifiers if identifiers else None
        )
    )
    for outpostIns in outpostQS:
        send_msg(outpostIns)


def create_id(serializer, data):
    idSerailizer = serializer(data=data)
    idSerailizer.is_valid(raise_exception=True)
    idSerailizer.save()


def update_id(identifier, model, serializer, data):
    instance = model.objects.get(identifier=identifier)
    serializedData = serializer(instance, data=data, partial=True)
    serializedData.is_valid(raise_exception=True)
    serializedData.save()


def delete_id(identifier, model):
    instance = model.objects.get(identifier=identifier)
    instance.delete()


def get_id_model_serializer(action):
    if action.endswith("_UID"):
        return UID, UIDViewModelSerializer
    elif action.endswith("_OID"):
        return OID, OIDViewModelSerializer
    elif action.endswith("_TID"):
        return TID, TIDViewModelSerializer
    elif action.endswith("_PID"):
        return PID, PIDViewModelSerializer
    elif action.endswith("_CID"):
        return CID, CIDViewModelSerializer


def process_id(requestData):
    action = DictUtils.get_by_path(requestData, "action")
    data = DictUtils.get_by_path(requestData, "data")
    identifier = DictUtils.get_by_path(requestData, "data.identifier")
    logger.info(
        "Recd {} command on msmq, request data: {}".format(
            action, json.dumps(requestData, indent=4)
        )
    )
    model, serializer = get_id_model_serializer(action)
    if action in [
        UID_ACTION_CREATE,
        OID_ACTION_CREATE,
        TID_ACTION_CREATE,
        PID_ACTION_CREATE,
        CID_ACTION_CREATE,
    ]:
        create_id(serializer, data)

    if action in [
        UID_ACTION_UPDATE,
        OID_ACTION_UPDATE,
        TID_ACTION_UPDATE,
        PID_ACTION_UPDATE,
        CID_ACTION_UPDATE,
    ]:
        update_id(identifier, model, serializer, data)

    if action in [
        UID_ACTION_DELETE,
        OID_ACTION_DELETE,
        TID_ACTION_DELETE,
        PID_ACTION_DELETE,
        CID_ACTION_DELETE,
    ]:
        delete_id(identifier, model)


callback_map = {
    OUTPOST_ACTION_CLEAR: clear_outpost,
    UID_ACTION_CREATE: process_id,
    OID_ACTION_CREATE: process_id,
    TID_ACTION_CREATE: process_id,
    PID_ACTION_CREATE: process_id,
    CID_ACTION_CREATE: process_id,
    UID_ACTION_UPDATE: process_id,
    OID_ACTION_UPDATE: process_id,
    TID_ACTION_UPDATE: process_id,
    PID_ACTION_UPDATE: process_id,
    CID_ACTION_UPDATE: process_id,
    UID_ACTION_DELETE: process_id,
    OID_ACTION_DELETE: process_id,
    TID_ACTION_DELETE: process_id,
    PID_ACTION_DELETE: process_id,
    CID_ACTION_DELETE: process_id,
}


def base_callback(ch, method, properties, body):
    bodyStr = body.decode().replace("'", '"')
    jsonStr = json.loads(bodyStr)
    serializedData = MsgSerializer(data=jsonStr)
    serializedData.is_valid(raise_exception=True)

    action = DictUtils.get(serializedData.data, "action")
    if action in callback_map:
        callback_map[action](serializedData.data)
    else:
        logger.error("Action : {} not handled in message consumption".format(action))


class Command(BaseCommand):
    help = "Listens to a RabbitMQ topic"

    def __init__(self, callbackFn=base_callback):
        self.__callbackFn = callbackFn

    def add_arguments(self, parser):
        parser.add_argument(
            "-s",
            "--service_config_id",
            type=str,
            help="Service Config Id",
        )

    def connectToMSMQ(self, serviceConfigId):
        try:
            msmqHandler = RabbitMQ.getSingleton(serviceConfigId)
            return msmqHandler
        except Exception as e:
            print(e)
            logger.error(
                "Error connecting to RabbitMQ, check if RabbitMQ instance is up!"
            )
            Time.sleep(30)
            self.connectToMSMQ()

    def handle(self, *args, **options):
        logger.info("Listening to MSMQ messages...")
        serviceConfigId = DictUtils.get(options, "service_config_id")
        msmqHandler = self.connectToMSMQ(serviceConfigId)
        msmqHandler.consumeQueues(self.__callbackFn)
        msmqHandler.startConsuming()



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.cid import CID
from kfsd.apps.endpoints.serializers.common.cid import CIDViewModelSerializer
from kfsd.apps.endpoints.views.docs.cid import CIDDoc


@extend_schema_view(**CIDDoc.modelviewset())
class CIDModelViewSet(CustomModelViewSet):
    queryset = CID.objects.all()
    serializer_class = CIDViewModelSerializer



MIN_LENGTH = 1
MAX_LENGTH = 400

IDENTIFIER_REGEX_CONDITION = r"^[A-Za-z0-9\.\-:_,=\/@]+$"
MSMQ_NAME_REGEX_CONDITION = r"^[A-Za-z0-9_\.]+$"
NAME_REGEX_CONDITION = r"^[A-Za-z0-9/\-_:,\s@\.]+$"


ORG = "ORG"
APPLICATION = "APP"
PROJECT = "PRJ"
TYPE = "TYPE"
NAMESPACE = "NS"

# SERVICE_CONFIG_ID
SERVICE_CONFIG_ID_REMIND = "remind"
SERVICE_CONFIG_ID_UTILS = "utils"
SERVICE_CONFIG_ID_SSO_FE = "sso_fe"
SERVICE_CONFIG_ID_SSO_API = "sso_api"
SERVICE_CONFIG_ID_GATEWAY = "gateway"
SERVICE_CONFIG_ID_CERTS = "certs"
SERVICE_CONFIG_ID_PERMS = "perms"

# COMMON ACTIONS
UID_ACTION_CREATE = "CREATE_UID"
UID_ACTION_DELETE = "DELETE_UID"
UID_ACTION_UPDATE = "UPDATE_UID"

OID_ACTION_CREATE = "CREATE_OID"
OID_ACTION_DELETE = "DELETE_OID"
OID_ACTION_UPDATE = "UPDATE_OID"

TID_ACTION_CREATE = "CREATE_TID"
TID_ACTION_DELETE = "DELETE_TID"
TID_ACTION_UPDATE = "UPDATE_TID"

PID_ACTION_CREATE = "CREATE_PID"
PID_ACTION_DELETE = "DELETE_PID"
PID_ACTION_UPDATE = "UPDATE_PID"

CID_ACTION_CREATE = "CREATE_CID"
CID_ACTION_DELETE = "DELETE_CID"
CID_ACTION_UPDATE = "UPDATE_CID"

OUTPOST_ACTION_CLEAR = "CLEAR_OUTPOST"
REMIND_ACTION_CREATE = "CREATE_REMINDER"
REMIND_ACTION_DELETE = "DELETE_REMINDER"
REMIND_ACTION_IN_MINS = 60



from drf_spectacular.utils import OpenApiExample, OpenApiTypes, OpenApiParameter


class UIDV1Doc:
    @staticmethod
    def modelviewset_list_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.QUERY,
                name="page",
                required=False,
                type=OpenApiTypes.INT,
                examples=[
                    OpenApiExample("Example 1", summary="Pagination", value=1),
                    OpenApiExample("Example 2", summary="Pagination", value=2),
                ],
            )
        ]

    @staticmethod
    def modelviewset_list_examples():
        return [
            OpenApiExample(
                "UID - List All",
                value=[
                    {
                        "identifier": "USER=admin@kubefacets.com",
                        "slug": "admin",
                        "name": "admin@kubefacets.com",
                    }
                ],
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_get_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "UID - Get",
                        summary="UID Identifier",
                        description="UID - Get",
                        value="USER=admin@kubefacets.com",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_get_examples():
        return [
            OpenApiExample(
                "UID - Get",
                value={
                    "identifier": "USER=admin@kubefacets.com",
                    "slug": "admin",
                    "name": "admin@kubefacets.com",
                },
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_create_examples():
        return [
            OpenApiExample(
                "UID - Create",
                value={
                    "identifier": "USER=admin@kubefacets.com",
                    "slug": "admin",
                    "name": "admin@kubefacets.com",
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "UID - Create",
                value={
                    "identifier": "USER=admin@kubefacets.com",
                    "slug": "admin",
                    "name": "admin@kubefacets.com",
                },
                request_only=False,
                response_only=True,
            ),
        ]

    @staticmethod
    def modelviewset_delete_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "UID - Delete",
                        summary="UID Identifier",
                        description="UID - Delete",
                        value="USER=admin@kubefacets.com",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "UID - Delete",
                        summary="UID Identifier",
                        description="UID - Delete",
                        value="USER=admin@kubefacets.com",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_examples():
        return [
            OpenApiExample(
                "UID - Patch",
                value={
                    "name": "admin1",
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "UID - Patch",
                value={
                    "identifier": "USER=admin@kubefacets.com",
                    "name": "admin@kubefacets.com",
                    "slug": "admin",
                },
                request_only=False,
                response_only=True,
            ),
        ]



from kfsd.apps.endpoints.serializers.common.id import IDModelSerializer
from kfsd.apps.models.tables.cid import CID


class CIDModelSerializer(IDModelSerializer):
    class Meta:
        model = CID
        fields = "__all__"


class CIDViewModelSerializer(CIDModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = CID
        exclude = ("created", "updated", "id")



from drf_spectacular.utils import OpenApiExample, OpenApiTypes, OpenApiParameter


class CIDV1Doc:
    @staticmethod
    def modelviewset_list_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.QUERY,
                name="page",
                required=False,
                type=OpenApiTypes.INT,
                examples=[
                    OpenApiExample("Example 1", summary="Pagination", value=1),
                    OpenApiExample("Example 2", summary="Pagination", value=2),
                ],
            )
        ]

    @staticmethod
    def modelviewset_list_examples():
        return [
            OpenApiExample(
                "CID - List All",
                value=[
                    {
                        "identifier": "ORG=Kubefacets Inc,PRJ=Config,COLL=SSO",
                        "slug": "sso",
                        "name": "SSO",
                    }
                ],
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_get_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "CID - Get",
                        summary="CID Identifier",
                        description="CID - Get",
                        value="ORG=Kubefacets Inc,PRJ=Config,COLL=SSO",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_get_examples():
        return [
            OpenApiExample(
                "CID - Get",
                value={
                    "identifier": "ORG=Kubefacets Inc,PRJ=Config,COLL=SSO",
                    "slug": "sso",
                    "name": "SSO",
                },
                request_only=False,
                response_only=True,
            )
        ]

    @staticmethod
    def modelviewset_create_examples():
        return [
            OpenApiExample(
                "CID - Create",
                value={
                    "identifier": "ORG=Kubefacets Inc,PRJ=Config,COLL=Cert",
                    "slug": "cert",
                    "name": "Cert",
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "CID - Create",
                value={
                    "identifier": "ORG=Kubefacets Inc,PRJ=Config,COLL=Cert",
                    "slug": "cert",
                    "name": "Cert",
                },
                request_only=False,
                response_only=True,
            ),
        ]

    @staticmethod
    def modelviewset_delete_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "CID - Delete",
                        summary="CID Identifier",
                        description="CID - Delete",
                        value="ORG=Kubefacets Inc,PRJ=Config,COLL=Cert",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_path_examples():
        return [
            OpenApiParameter(
                location=OpenApiParameter.PATH,
                name="identifier",
                required=True,
                type=OpenApiTypes.STR,
                examples=[
                    OpenApiExample(
                        "CID - Patch",
                        summary="CID Identifier",
                        description="CID - Patch",
                        value="ORG=Kubefacets Inc,PRJ=Config,COLL=Cert",
                    )
                ],
            )
        ]

    @staticmethod
    def modelviewset_patch_examples():
        return [
            OpenApiExample(
                "CID - Patch",
                value={
                    "slug": "certs",
                },
                request_only=True,
                response_only=False,
            ),
            OpenApiExample(
                "TID - Patch",
                value={
                    "identifier": "ORG=Kubefacets Inc,PRJ=Config,COLL=Cert",
                    "slug": "certs",
                    "name": "Cert",
                },
                request_only=False,
                response_only=True,
            ),
        ]



from kfsd.apps.models.tables.id import ID


# Organization Reference Table
class OID(ID):
    class Meta:
        app_label = "models"
        verbose_name = "OID"
        verbose_name_plural = "OID"



from django.urls import reverse
from rest_framework import status
from unittest.mock import patch
from kfsd.apps.endpoints.tests.endpoints_test_handler import EndpointsTestHandler


class PIDEndpointTestCases(EndpointsTestHandler):
    fixtures = ["v1/tests/common/ids.json"]

    def setUp(self):
        self.__pidListUrl = reverse("pid-list")
        super().setUp()

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_list_pid(self, remoteConfigMocked, rawSettingsMocked, tokenUserMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig

        currentPg = 1
        obsResponse = self.list(self.__pidListUrl, currentPg, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/responses/test_list_pid.json"
        )
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_get_pid(self, remoteConfigMocked, rawSettingsMocked, tokenUserMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig

        dataIdentifier = "ORG=Kubefacets Inc,PRJ=Config"
        obsResponse = self.get("pid-detail", dataIdentifier, status.HTTP_200_OK)
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/responses/test_get_pid.json"
        )
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_create_pid(self, remoteConfigMocked, rawSettingsMocked, tokenUserMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig

        requestData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/test_create_pid.json"
        )
        self.__obsResponse = self.create(
            self.__pidListUrl, requestData, status.HTTP_201_CREATED
        )

        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/responses/test_create_pid.json"
        )
        self.assertEqual(self.__obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_patch_pid(self, remoteConfigMocked, rawSettingsMocked, tokenUserMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig

        # Create Test data
        requestData = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/test_create_pid.json"
        )
        self.__obsResponse = self.create(
            self.__pidListUrl, requestData, status.HTTP_201_CREATED
        )

        # Test Patch
        identifier = "ORG=Kubefacets Inc,PRJ=Config1"
        requestData = {"slug": "config2"}
        obsResponse = self.patch(
            "pid-detail", identifier, requestData, status.HTTP_200_OK
        )
        expResponse = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/responses/test_patch_pid.json"
        )
        self.assertEqual(obsResponse, expResponse)

    @patch("kfsd.apps.core.auth.api.token.TokenAuth.getTokenUserInfo")
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.getLocalKubefacetsSettingsConfig"
    )
    @patch(
        "kfsd.apps.core.common.kubefacets_config.KubefacetsConfig.deriveRemoteConfig"
    )
    def test_delete_pid(self, remoteConfigMocked, rawSettingsMocked, tokenUserMocked):
        rawConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_settings.json"
        )
        remoteConfig = self.readJSONData(
            "kfsd/apps/endpoints/tests/v1/common/data/requests/kubefacets/remote_config_resp.json"
        )
        rawSettingsMocked.return_value = rawConfig
        remoteConfigMocked.return_value = remoteConfig

        identifier = "ORG=Kubefacets Inc,PRJ=Config"
        self.delete("pid-detail", identifier, status.HTTP_204_NO_CONTENT)
        # should return not found as it's already deleted
        self.delete("pid-detail", identifier, status.HTTP_404_NOT_FOUND)



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.oid import OID
from kfsd.apps.endpoints.serializers.common.oid import OIDViewModelSerializer
from kfsd.apps.endpoints.views.docs.oid import OIDDoc


@extend_schema_view(**OIDDoc.modelviewset())
class OIDModelViewSet(CustomModelViewSet):
    queryset = OID.objects.all()
    serializer_class = OIDViewModelSerializer



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.tid import TID
from kfsd.apps.endpoints.serializers.common.tid import TIDViewModelSerializer
from kfsd.apps.endpoints.views.docs.tid import TIDDoc


@extend_schema_view(**TIDDoc.modelviewset())
class TIDModelViewSet(CustomModelViewSet):
    queryset = TID.objects.all()
    serializer_class = TIDViewModelSerializer



from drf_spectacular.utils import extend_schema_view

from kfsd.apps.endpoints.views.common.custom_model import CustomModelViewSet
from kfsd.apps.models.tables.pid import PID
from kfsd.apps.endpoints.serializers.common.pid import PIDViewModelSerializer
from kfsd.apps.endpoints.views.docs.pid import PIDDoc


@extend_schema_view(**PIDDoc.modelviewset())
class PIDModelViewSet(CustomModelViewSet):
    queryset = PID.objects.all()
    serializer_class = PIDViewModelSerializer



from rest_framework import serializers
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
    RegexValidator,
)
from django.utils.translation import gettext_lazy as _

from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH
from kfsd.apps.models.constants import NAME_REGEX_CONDITION
from kfsd.apps.endpoints.serializers.common.model import BaseModelSerializer


class IDModelSerializer(BaseModelSerializer):
    identifier = serializers.CharField(read_only=False)
    slug = serializers.SlugField(read_only=False)
    name = serializers.CharField(
        required=True,
        validators=[
            MinLengthValidator(MIN_LENGTH),
            MaxLengthValidator(MAX_LENGTH),
            RegexValidator(
                NAME_REGEX_CONDITION,
                message=_(
                    "Only alpha numeric and spaces are allowed, regex: {}".format(
                        NAME_REGEX_CONDITION
                    )
                ),
                code="invalid_name",
            ),
        ],
    )
    attrs = serializers.JSONField(default=dict)



from django.urls import path, include
from drf_spectacular.views import SpectacularAPIView
from rest_framework import routers
from kfsd.apps.endpoints.views.common.cid import CIDModelViewSet
from kfsd.apps.endpoints.views.common.pid import PIDModelViewSet
from kfsd.apps.endpoints.views.common.tid import TIDModelViewSet
from kfsd.apps.endpoints.views.common.uid import UIDModelViewSet
from kfsd.apps.endpoints.views.common.oid import OIDModelViewSet
from kfsd.apps.endpoints.views.utils.utils import UtilsViewSet
from kfsd.apps.endpoints.views.utils.common import CommonViewSet
from kfsd.apps.endpoints.views.common.outpost import OutpostModelViewSet

router = routers.DefaultRouter()
router.include_format_suffixes = False

router.register("cids", CIDModelViewSet, basename="cid")
router.register("pids", PIDModelViewSet, basename="pid")
router.register("tids", TIDModelViewSet, basename="tid")
router.register("oids", OIDModelViewSet, basename="oid")
router.register("uids", UIDModelViewSet, basename="uid")
router.register("utils", UtilsViewSet, basename="utils")
router.register("common", CommonViewSet, basename="common")
router.register("outpost", OutpostModelViewSet, basename="outpost")

urlpatterns = [
    path("", include(router.urls)),
    path("schema/", SpectacularAPIView.as_view(), name="schema-api"),
]



from django.db import models
from django.core.validators import (
    MinLengthValidator,
    MaxLengthValidator,
    RegexValidator,
)

from kfsd.apps.models.constants import MAX_LENGTH, MIN_LENGTH
from kfsd.apps.models.tables.base import BaseModel
from kfsd.apps.models.constants import NAME_REGEX_CONDITION


class Feature(BaseModel):
    class Meta:
        abstract = True

    name = models.CharField(
        max_length=MAX_LENGTH,
        validators=[
            RegexValidator(
                regex=NAME_REGEX_CONDITION,
                message="name doesnt match condition {}".format(NAME_REGEX_CONDITION),
            ),
            MaxLengthValidator(MAX_LENGTH),
            MinLengthValidator(MIN_LENGTH),
        ],
    )
    slug = models.SlugField()
    description = models.TextField(blank=True)
    creator = models.ForeignKey("UID", on_delete=models.PROTECT, null=True, blank=True)



from kfsd.apps.endpoints.serializers.common.id import IDModelSerializer
from kfsd.apps.models.tables.tid import TID


class TIDModelSerializer(IDModelSerializer):
    class Meta:
        model = TID
        fields = "__all__"


class TIDViewModelSerializer(TIDModelSerializer):
    id = None
    created = None
    updated = None

    class Meta:
        model = TID
        exclude = ("created", "updated", "id")
